/*
CHANGES:
1. pageRow property was introduced and it binds to
   applicationWindow().activityStack.currentActivity.pageRow (line 58)
2. onPageRowChanged signal handler was introduced (line 66)
3. all references to applicationWindow().pageStack were changed to point to local pageRow property
   (previous versions has been removed to keep the code readable)
*/


/*
 *   Copyright 2015 Marco Martin <mart@kde.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2 or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Library General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

import QtQuick 2.5
import QtQuick.Layouts 1.2
import "private"
import org.kde.kirigami 2.0


/**
 * An item that can be used as a title for the application.
 * Scrolling the main page will make it taller or shorter (trough the point of going away)
 * It's a behavior similar to the typical mobile web browser adressbar
 * the minimum, preferred and maximum heights of the item can be controlled with
 * * minimumHeight: default is 0, i.e. hidden
 * * preferredHeight: default is Units.gridUnit * 1.6
 * * preferredHeight: default is Units.gridUnit * 3
 *
 * To achieve a titlebar that stays completely fixed just set the 3 sizes as the same
 */
Item {
    id: root
    z: 90
    property int minimumHeight: 0
    property int preferredHeight: Units.gridUnit * 2
    property int maximumHeight: Units.gridUnit * 3
    default property alias contentItem: mainItem.data
    readonly property int paintedHeight: headerItem.y + headerItem.height - 1

    //FIXME: remove
    property QtObject __appWindow: applicationWindow();

    property Item pageRow: __appWindow.activityStack
                           && __appWindow.activityStack.currentActivity
                           && __appWindow.activityStack.currentActivity.pageRow ?
                                      __appWindow.activityStack.currentActivity.pageRow : null

    // pageRow property binds to current activity's pageRow, so if current activity changes
    // so does pageRow. The header could be hidden in previous activity so its important
    // to reset its position on activity change
    onPageRowChanged: {
        headerItem.y = 0;
        headerItem.updatePageHeader()
    }

    anchors {
        left: parent.left
        right: parent.right
    }
    height: {
        if (!__appWindow.controlsVisible) {
            return 1;
        } else if (__appWindow.wideScreen) {
            return preferredHeight;
        } else {
            return 1;
        }
    }

    /**
     * background: Item
     * This property holds the background item.
     * Note: the background will be automatically sized as the whole control
     */
    property Item background

    onBackgroundChanged: {
        background.z = -1;
        background.parent = headerItem;
        background.anchors.fill = headerItem;
    }

    opacity: height > 0 && -translateTransform.y <= height ? 1 : 0
    Behavior on opacity {
        OpacityAnimator {
            duration: Units.longDuration
            easing.type: Easing.InOutQuad
        }
    }

    transform: Translate {
        id: translateTransform
        y: {
            if (__appWindow === undefined) {
                return 0;
            }
            if (!__appWindow.controlsVisible) {
                return -headerItem.height - Units.smallSpacing;
            } else {
                return 0;
            }
        }
        Behavior on y {
            NumberAnimation {
                duration: Units.longDuration
                easing.type: translateTransform.y < 0 ? Easing.OutQuad : Easing.InQuad
            }
        }
    }

    Item {
        id: headerItem
        anchors {
            left: parent.left
            right: parent.right
        }

        height: __appWindow.reachableMode ? root.maximumHeight : root.preferredHeight

        function updatePageHeader() {
            if (!__appWindow || !pageRow || !pageRow.currentItem || !pageRow.currentItem.header || !pageRow.currentItem.flickable) {
                return;
            }

            if (__appWindow.wideScreen) {
                pageRow.currentItem.header.y = 0;
            } else {
                pageRow.currentItem.header.y = headerItem.height + headerItem.y -1;
            }
        }
        onYChanged: updatePageHeader()
        onHeightChanged: updatePageHeader()

        Connections {
            id: headerSlideConnection
            target: pageRow && pageRow.currentItem ? pageRow.currentItem.flickable : null
            property int oldContentY
            onContentYChanged: {
                if (!pageRow.currentItem) {
                    return;
                }
                if (pageRow.currentItem.flickable.atYBeginning ||
                    pageRow.currentItem.flickable.atYEnd) {
                    return;
                }

                if (__appWindow.wideScreen) {
                    headerItem.y = 0;
                } else {
                   // headerItem.y = Math.max(root.minimumHeight - root.preferredHeight, Math.min(0, headerItem.y + oldContentY - __appWindow.pageStack.currentItem.flickable.contentY));
                    headerItem.y = Math.max(root.minimumHeight - root.preferredHeight, Math.min(0, headerItem.y + oldContentY - pageRow.currentItem.flickable.contentY));
 
                    oldContentY = pageRow.currentItem.flickable.contentY;
                }
            }
            onMovementEnded: {
                if (headerItem.y > root.preferredHeight) {
                    //if don't change the position if more then preferredSize is shown
                } else if (headerItem.y < -(root.preferredHeight - root.minimumHeight)/2 ) {
                    headerItem.y = root.minimumHeight - root.preferredHeight;
                } else {
                    headerItem.y = 0;
                }
            }
        }
        Connections {
            target: pageRow ? pageRow : null
            onCurrentItemChanged: {
                if (!pageRow.currentItem) {
                    return;
                }
                if (pageRow.currentItem.flickable) {
                    headerSlideConnection.oldContentY = pageRow.currentItem.flickable.contentY;
                } else {
                    headerSlideConnection.oldContentY = 0;
                }
                headerItem.y = 0;
                headerItem.updatePageHeader()
            }
        }

        Item {
            id: mainItem
            anchors {
                fill: parent
            }
        }
        Behavior on y {
            enabled: pageRow && pageRow.currentItem && pageRow.currentItem.flickable && !pageRow.currentItem.flickable.moving
            NumberAnimation {
                duration: Units.longDuration
                easing.type: Easing.InOutQuad
            }
        }
    }


}

